@@ -1,11 +1,12 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package ceph
 
 import (
 	"bytes"
+	_ "embed"
 	"encoding/json"
 	"fmt"
-	"io/ioutil"
-	"log"
+	"os"
 	"os/exec"
 	"path/filepath"
 	"strings"
@@ -14,6 +15,10 @@ import (
 	"github.com/influxdata/telegraf/plugins/inputs"
 )
 
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//go:embed sample.conf
+var sampleConfig string
+
 const (
 	measurement = "ceph"
 	typeMon     = "monitor"
@@ -28,59 +33,22 @@ const (
 )
 
 type Ceph struct {
-	CephBinary             string
-	OsdPrefix              string
-	MonPrefix              string
-	MdsPrefix              string
-	RgwPrefix              string
-	SocketDir              string
-	SocketSuffix           string
-	CephUser               string
-	CephConfig             string
-	GatherAdminSocketStats bool
-	GatherClusterStats     bool
+	CephBinary             string `toml:"ceph_binary"`
+	OsdPrefix              string `toml:"osd_prefix"`
+	MonPrefix              string `toml:"mon_prefix"`
+	MdsPrefix              string `toml:"mds_prefix"`
+	RgwPrefix              string `toml:"rgw_prefix"`
+	SocketDir              string `toml:"socket_dir"`
+	SocketSuffix           string `toml:"socket_suffix"`
+	CephUser               string `toml:"ceph_user"`
+	CephConfig             string `toml:"ceph_config"`
+	GatherAdminSocketStats bool   `toml:"gather_admin_socket_stats"`
+	GatherClusterStats     bool   `toml:"gather_cluster_stats"`
+
+	Log telegraf.Logger `toml:"-"`
 }
 
-func (c *Ceph) Description() string {
-	return "Collects performance metrics from the MON, OSD, MDS and RGW nodes in a Ceph storage cluster."
-}
-
-var sampleConfig = `
-  ## This is the recommended interval to poll.  Too frequent and you will lose
-  ## data points due to timeouts during rebalancing and recovery
-  interval = '1m'
-
-  ## All configuration values are optional, defaults are shown below
-
-  ## location of ceph binary
-  ceph_binary = "/usr/bin/ceph"
-
-  ## directory in which to look for socket files
-  socket_dir = "/var/run/ceph"
-
-  ## prefix of MON and OSD socket files, used to determine socket type
-  mon_prefix = "ceph-mon"
-  osd_prefix = "ceph-osd"
-  mds_prefix = "ceph-mds"
-  rgw_prefix = "ceph-client"
-
-  ## suffix used to identify socket files
-  socket_suffix = "asok"
-
-  ## Ceph user to authenticate as
-  ceph_user = "client.admin"
-
-  ## Ceph configuration to use to locate the cluster
-  ceph_config = "/etc/ceph/ceph.conf"
-
-  ## Whether to gather statistics via the admin socket
-  gather_admin_socket_stats = true
-
-  ## Whether to gather statistics via ceph commands
-  gather_cluster_stats = false
-`
-
-func (c *Ceph) SampleConfig() string {
+func (*Ceph) SampleConfig() string {
 	return sampleConfig
 }
 
@@ -112,14 +80,14 @@ func (c *Ceph) gatherAdminSocketStats(acc telegraf.Accumulator) error {
 			acc.AddError(fmt.Errorf("error reading from socket '%s': %v", s.socket, err))
 			continue
 		}
-		data, err := parseDump(dump)
+		data, err := c.parseDump(dump)
 		if err != nil {
 			acc.AddError(fmt.Errorf("error parsing dump from socket '%s': %v", s.socket, err))
 			continue
 		}
 		for tag, metrics := range data {
 			acc.AddFields(measurement,
-				map[string]interface{}(metrics),
+				metrics,
 				map[string]string{"type": s.sockType, "id": s.sockID, "collection": tag})
 		}
 	}
@@ -138,7 +106,7 @@ func (c *Ceph) gatherClusterStats(acc telegraf.Accumulator) error {
 
 	// For each job, execute against the cluster, parse and accumulate the data points
 	for _, job := range jobs {
-		output, err := c.exec(job.command)
+		output, err := c.execute(job.command)
 		if err != nil {
 			return fmt.Errorf("error executing command: %v", err)
 		}
@@ -152,34 +120,36 @@ func (c *Ceph) gatherClusterStats(acc telegraf.Accumulator) error {
 }
 
 func init() {
-	c := Ceph{
-		CephBinary:             "/usr/bin/ceph",
-		OsdPrefix:              osdPrefix,
-		MonPrefix:              monPrefix,
-		MdsPrefix:              mdsPrefix,
-		RgwPrefix:              rgwPrefix,
-		SocketDir:              "/var/run/ceph",
-		SocketSuffix:           sockSuffix,
-		CephUser:               "client.admin",
-		CephConfig:             "/etc/ceph/ceph.conf",
-		GatherAdminSocketStats: true,
-		GatherClusterStats:     false,
-	}
-
-	inputs.Add(measurement, func() telegraf.Input { return &c })
+	inputs.Add(measurement, func() telegraf.Input {
+		return &Ceph{
+			CephBinary:             "/usr/bin/ceph",
+			OsdPrefix:              osdPrefix,
+			MonPrefix:              monPrefix,
+			MdsPrefix:              mdsPrefix,
+			RgwPrefix:              rgwPrefix,
+			SocketDir:              "/var/run/ceph",
+			SocketSuffix:           sockSuffix,
+			CephUser:               "client.admin",
+			CephConfig:             "/etc/ceph/ceph.conf",
+			GatherAdminSocketStats: true,
+			GatherClusterStats:     false,
+		}
+	})
 }
 
 var perfDump = func(binary string, socket *socket) (string, error) {
 	cmdArgs := []string{"--admin-daemon", socket.socket}
-	if socket.sockType == typeOsd {
+
+	switch socket.sockType {
+	case typeOsd:
 		cmdArgs = append(cmdArgs, "perf", "dump")
-	} else if socket.sockType == typeMon {
+	case typeMon:
 		cmdArgs = append(cmdArgs, "perfcounters_dump")
-	} else if socket.sockType == typeMds {
+	case typeMds:
 		cmdArgs = append(cmdArgs, "perf", "dump")
-	} else if socket.sockType == typeRgw {
+	case typeRgw:
 		cmdArgs = append(cmdArgs, "perf", "dump")
-	} else {
+	default:
 		return "", fmt.Errorf("ignoring unknown socket type: %s", socket.sockType)
 	}
 
@@ -195,7 +165,7 @@ var perfDump = func(binary string, socket *socket) (string, error) {
 }
 
 var findSockets = func(c *Ceph) ([]*socket, error) {
-	listing, err := ioutil.ReadDir(c.SocketDir)
+	listing, err := os.ReadDir(c.SocketDir)
 	if err != nil {
 		return []*socket{}, fmt.Errorf("Failed to read socket directory '%s': %v", c.SocketDir, err)
 	}
@@ -211,17 +181,14 @@ var findSockets = func(c *Ceph) ([]*socket, error) {
 		if strings.HasPrefix(f, c.OsdPrefix) {
 			sockType = typeOsd
 			sockPrefix = osdPrefix
-
 		}
 		if strings.HasPrefix(f, c.MdsPrefix) {
 			sockType = typeMds
 			sockPrefix = mdsPrefix
-
 		}
 		if strings.HasPrefix(f, c.RgwPrefix) {
 			sockType = typeRgw
 			sockPrefix = rgwPrefix
-
 		}
 
 		if sockType == typeOsd || sockType == typeMon || sockType == typeMds || sockType == typeRgw {
@@ -256,8 +223,10 @@ func (m *metric) name() string {
 	buf := bytes.Buffer{}
 	for i := len(m.pathStack) - 1; i >= 0; i-- {
 		if buf.Len() > 0 {
+			//nolint:errcheck,revive // should never return an error
 			buf.WriteString(".")
 		}
+		//nolint:errcheck,revive // should never return an error
 		buf.WriteString(m.pathStack[i])
 	}
 	return buf.String()
@@ -269,23 +238,23 @@ type taggedMetricMap map[string]metricMap
 
 // Parses a raw JSON string into a taggedMetricMap
 // Delegates the actual parsing to newTaggedMetricMap(..)
-func parseDump(dump string) (taggedMetricMap, error) {
+func (c *Ceph) parseDump(dump string) (taggedMetricMap, error) {
 	data := make(map[string]interface{})
 	err := json.Unmarshal([]byte(dump), &data)
 	if err != nil {
 		return nil, fmt.Errorf("failed to parse json: '%s': %v", dump, err)
 	}
 
-	return newTaggedMetricMap(data), nil
+	return c.newTaggedMetricMap(data), nil
 }
 
 // Builds a TaggedMetricMap out of a generic string map.
 // The top-level key is used as a tag and all sub-keys are flattened into metrics
-func newTaggedMetricMap(data map[string]interface{}) taggedMetricMap {
+func (c *Ceph) newTaggedMetricMap(data map[string]interface{}) taggedMetricMap {
 	tmm := make(taggedMetricMap)
 	for tag, datapoints := range data {
 		mm := make(metricMap)
-		for _, m := range flatten(datapoints) {
+		for _, m := range c.flatten(datapoints) {
 			mm[m.name()] = m.value
 		}
 		tmm[tag] = mm
@@ -297,29 +266,33 @@ func newTaggedMetricMap(data map[string]interface{}) taggedMetricMap {
 // Nested keys are flattened into ordered slices associated with a metric value.
 // The key slices are treated as stacks, and are expected to be reversed and concatenated
 // when passed as metrics to the accumulator. (see (*metric).name())
-func flatten(data interface{}) []*metric {
+func (c *Ceph) flatten(data interface{}) []*metric {
 	var metrics []*metric
 
 	switch val := data.(type) {
 	case float64:
-		metrics = []*metric{{make([]string, 0, 1), val}}
+		metrics = []*metric{
+			{
+				make([]string, 0, 1), val,
+			},
+		}
 	case map[string]interface{}:
 		metrics = make([]*metric, 0, len(val))
 		for k, v := range val {
-			for _, m := range flatten(v) {
+			for _, m := range c.flatten(v) {
 				m.pathStack = append(m.pathStack, k)
 				metrics = append(metrics, m)
 			}
 		}
 	default:
-		log.Printf("I! [inputs.ceph] ignoring unexpected type '%T' for value %v", val, val)
+		c.Log.Infof("ignoring unexpected type '%T' for value %v", val, val)
 	}
 
 	return metrics
 }
 
-// exec executes the 'ceph' command with the supplied arguments, returning JSON formatted output
-func (c *Ceph) exec(command string) (string, error) {
+// execute executes the 'ceph' command with the supplied arguments, returning JSON formatted output
+func (c *Ceph) execute(command string) (string, error) {
 	cmdArgs := []string{"--conf", c.CephConfig, "--name", c.CephUser, "--format", "json"}
 	cmdArgs = append(cmdArgs, strings.Split(command, " ")...)
 
@@ -336,8 +309,8 @@ func (c *Ceph) exec(command string) (string, error) {
 
 	// Ceph doesn't sanitize its output, and may return invalid JSON.  Patch this
 	// up for them, as having some inaccurate data is better than none.
-	output = strings.Replace(output, "-inf", "0", -1)
-	output = strings.Replace(output, "inf", "0", -1)
+	output = strings.ReplaceAll(output, "-inf", "0")
+	output = strings.ReplaceAll(output, "inf", "0")
 
 	return output, nil
 }
