@@ -112,12 +112,7 @@ func TestGather(t *testing.T) {
 }
 
 func TestFindSockets(t *testing.T) {
-	tmpdir, err := os.MkdirTemp("", "socktest")
-	require.NoError(t, err)
-	defer func() {
-		err := os.Remove(tmpdir)
-		require.NoError(t, err)
-	}()
+	tmpdir := t.TempDir()
 	c := &Ceph{
 		CephBinary:             "foo",
 		OsdPrefix:              "ceph-osd",
@@ -188,7 +183,7 @@ func createTestFiles(dir string, st *SockTest) error {
 	writeFile := func(prefix string, i int) error {
 		f := sockFile(prefix, i)
 		fpath := filepath.Join(dir, f)
-		return os.WriteFile(fpath, []byte(""), 0777)
+		return os.WriteFile(fpath, []byte(""), 0644)
 	}
 	return tstFileApply(st, writeFile)
 }
