@@ -1,9 +1,11 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package directory_monitor
 
 import (
 	"bufio"
 	"compress/gzip"
 	"context"
+	_ "embed"
 	"errors"
 	"fmt"
 	"io"
@@ -13,61 +15,21 @@ import (
 	"sync"
 	"time"
 
+	"github.com/djherbis/times"
 	"golang.org/x/sync/semaphore"
-	"gopkg.in/djherbis/times.v1"
 
 	"github.com/influxdata/telegraf"
 	"github.com/influxdata/telegraf/config"
+	"github.com/influxdata/telegraf/internal/choice"
 	"github.com/influxdata/telegraf/plugins/inputs"
 	"github.com/influxdata/telegraf/plugins/parsers"
 	"github.com/influxdata/telegraf/plugins/parsers/csv"
 	"github.com/influxdata/telegraf/selfstat"
 )
 
-const sampleConfig = `
-  ## The directory to monitor and read files from.
-  directory = ""
-  #
-  ## The directory to move finished files to.
-  finished_directory = ""
-  #
-  ## The directory to move files to upon file error.
-  ## If not provided, erroring files will stay in the monitored directory.
-  # error_directory = ""
-  #
-  ## The amount of time a file is allowed to sit in the directory before it is picked up.
-  ## This time can generally be low but if you choose to have a very large file written to the directory and it's potentially slow,
-  ## set this higher so that the plugin will wait until the file is fully copied to the directory.
-  # directory_duration_threshold = "50ms"
-  #
-  ## A list of the only file names to monitor, if necessary. Supports regex. If left blank, all files are ingested.
-  # files_to_monitor = ["^.*\.csv"]
-  #
-  ## A list of files to ignore, if necessary. Supports regex.
-  # files_to_ignore = [".DS_Store"]
-  #
-  ## Maximum lines of the file to process that have not yet be written by the
-  ## output. For best throughput set to the size of the output's metric_buffer_limit.
-  ## Warning: setting this number higher than the output's metric_buffer_limit can cause dropped metrics.
-  # max_buffered_metrics = 10000
-  #
-  ## The maximum amount of file paths to queue up for processing at once, before waiting until files are processed to find more files.
-  ## Lowering this value will result in *slightly* less memory use, with a potential sacrifice in speed efficiency, if absolutely necessary.
-  #	file_queue_size = 100000
-  #
-  ## Name a tag containing the name of the file the data was parsed from.  Leave empty
-  ## to disable. Cautious when file name variation is high, this can increase the cardinality 
-  ## significantly. Read more about cardinality here: 
-  ## https://docs.influxdata.com/influxdb/cloud/reference/glossary/#series-cardinality
-  # file_tag = ""
-  #
-  ## The dataformat to be read from the files.
-  ## Each data format has its own unique set of configuration options, read
-  ## more about them here:
-  ## https://github.com/influxdata/telegraf/blob/master/docs/DATA_FORMATS_INPUT.md
-  ## NOTE: We currently only support parsing newline-delimited JSON. See the format here: https://github.com/ndjson/ndjson-spec
-  data_format = "influx"
-`
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//go:embed sample.conf
+var sampleConfig string
 
 var (
 	defaultFilesToMonitor             = []string{}
@@ -75,6 +37,7 @@ var (
 	defaultMaxBufferedMetrics         = 10000
 	defaultDirectoryDurationThreshold = config.Duration(0 * time.Millisecond)
 	defaultFileQueueSize              = 100000
+	defaultParseMethod                = "line-by-line"
 )
 
 type DirectoryMonitor struct {
@@ -89,6 +52,7 @@ type DirectoryMonitor struct {
 	DirectoryDurationThreshold config.Duration `toml:"directory_duration_threshold"`
 	Log                        telegraf.Logger `toml:"-"`
 	FileQueueSize              int             `toml:"file_queue_size"`
+	ParseMethod                string          `toml:"parse_method"`
 
 	filesInUse          sync.Map
 	cancel              context.CancelFunc
@@ -104,14 +68,10 @@ type DirectoryMonitor struct {
 	filesToProcess      chan string
 }
 
-func (monitor *DirectoryMonitor) SampleConfig() string {
+func (*DirectoryMonitor) SampleConfig() string {
 	return sampleConfig
 }
 
-func (monitor *DirectoryMonitor) Description() string {
-	return "Ingests files in a directory and then moves them to a target directory."
-}
-
 func (monitor *DirectoryMonitor) Gather(_ telegraf.Accumulator) error {
 	// Get all files sitting in the directory.
 	files, err := os.ReadDir(monitor.Directory)
@@ -243,7 +203,7 @@ func (monitor *DirectoryMonitor) ingestFile(filePath string) error {
 
 	parser, err := monitor.parserFunc()
 	if err != nil {
-		return fmt.Errorf("E! Creating parser: %s", err.Error())
+		return fmt.Errorf("creating parser: %w", err)
 	}
 
 	// Handle gzipped files.
@@ -261,41 +221,70 @@ func (monitor *DirectoryMonitor) ingestFile(filePath string) error {
 }
 
 func (monitor *DirectoryMonitor) parseFile(parser parsers.Parser, reader io.Reader, fileName string) error {
+	var splitter bufio.SplitFunc
+
+	// Decide on how to split the file
+	switch monitor.ParseMethod {
+	case "at-once":
+		return monitor.parseAtOnce(parser, reader, fileName)
+	case "line-by-line":
+		splitter = bufio.ScanLines
+	default:
+		return fmt.Errorf("unknown parse method %q", monitor.ParseMethod)
+	}
+
 	scanner := bufio.NewScanner(reader)
+	scanner.Split(splitter)
+
 	for scanner.Scan() {
-		metrics, err := monitor.parseLine(parser, scanner.Bytes())
+		metrics, err := monitor.parseMetrics(parser, scanner.Bytes(), fileName)
 		if err != nil {
 			return err
 		}
 
-		if monitor.FileTag != "" {
-			for _, m := range metrics {
-				m.AddTag(monitor.FileTag, filepath.Base(fileName))
-			}
-		}
-
 		if err := monitor.sendMetrics(metrics); err != nil {
 			return err
 		}
 	}
 
-	return nil
+	return scanner.Err()
 }
 
-func (monitor *DirectoryMonitor) parseLine(parser parsers.Parser, line []byte) ([]telegraf.Metric, error) {
+func (monitor *DirectoryMonitor) parseAtOnce(parser parsers.Parser, reader io.Reader, fileName string) error {
+	bytes, err := io.ReadAll(reader)
+	if err != nil {
+		return err
+	}
+
+	metrics, err := monitor.parseMetrics(parser, bytes, fileName)
+	if err != nil {
+		return err
+	}
+
+	return monitor.sendMetrics(metrics)
+}
+
+func (monitor *DirectoryMonitor) parseMetrics(parser parsers.Parser, line []byte, fileName string) (metrics []telegraf.Metric, err error) {
 	switch parser.(type) {
 	case *csv.Parser:
-		m, err := parser.Parse(line)
+		metrics, err = parser.Parse(line)
 		if err != nil {
 			if errors.Is(err, io.EOF) {
 				return nil, nil
 			}
 			return nil, err
 		}
-		return m, err
 	default:
-		return parser.Parse(line)
+		metrics, err = parser.Parse(line)
 	}
+
+	if monitor.FileTag != "" {
+		for _, m := range metrics {
+			m.AddTag(monitor.FileTag, filepath.Base(fileName))
+		}
+	}
+
+	return metrics, err
 }
 
 func (monitor *DirectoryMonitor) sendMetrics(metrics []telegraf.Metric) error {
@@ -359,7 +348,7 @@ func (monitor *DirectoryMonitor) Init() error {
 
 	// Finished directory can be created if not exists for convenience.
 	if _, err := os.Stat(monitor.FinishedDirectory); os.IsNotExist(err) {
-		err = os.Mkdir(monitor.FinishedDirectory, 0777)
+		err = os.Mkdir(monitor.FinishedDirectory, 0755)
 		if err != nil {
 			return err
 		}
@@ -371,7 +360,7 @@ func (monitor *DirectoryMonitor) Init() error {
 	// If an error directory should be used but has not been configured yet, create one ourselves.
 	if monitor.ErrorDirectory != "" {
 		if _, err := os.Stat(monitor.ErrorDirectory); os.IsNotExist(err) {
-			err := os.Mkdir(monitor.ErrorDirectory, 0777)
+			err := os.Mkdir(monitor.ErrorDirectory, 0755)
 			if err != nil {
 				return err
 			}
@@ -400,6 +389,10 @@ func (monitor *DirectoryMonitor) Init() error {
 		monitor.fileRegexesToIgnore = append(monitor.fileRegexesToIgnore, regex)
 	}
 
+	if err := choice.Check(monitor.ParseMethod, []string{"line-by-line", "at-once"}); err != nil {
+		return fmt.Errorf("config option parse_method: %w", err)
+	}
+
 	return nil
 }
 
@@ -411,6 +404,7 @@ func init() {
 			MaxBufferedMetrics:         defaultMaxBufferedMetrics,
 			DirectoryDurationThreshold: defaultDirectoryDurationThreshold,
 			FileQueueSize:              defaultFileQueueSize,
+			ParseMethod:                defaultParseMethod,
 		}
 	})
 }
