@@ -1,9 +1,11 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package elasticsearch
 
 import (
+	_ "embed"
 	"encoding/json"
 	"fmt"
-	"io/ioutil"
+	"io"
 	"net/http"
 	"regexp"
 	"sort"
@@ -19,6 +21,10 @@ import (
 	jsonparser "github.com/influxdata/telegraf/plugins/parsers/json"
 )
 
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//go:embed sample.conf
+var sampleConfig string
+
 // mask for masking username/password from error messages
 var mask = regexp.MustCompile(`https?:\/\/\S+:\S+@`)
 
@@ -86,64 +92,6 @@ type indexStat struct {
 	Shards    map[string][]interface{} `json:"shards"`
 }
 
-const sampleConfig = `
-  ## specify a list of one or more Elasticsearch servers
-  # you can add username and password to your url to use basic authentication:
-  # servers = ["http://user:pass@localhost:9200"]
-  servers = ["http://localhost:9200"]
-
-  ## Timeout for HTTP requests to the elastic search server(s)
-  http_timeout = "5s"
-
-  ## When local is true (the default), the node will read only its own stats.
-  ## Set local to false when you want to read the node stats from all nodes
-  ## of the cluster.
-  local = true
-
-  ## Set cluster_health to true when you want to also obtain cluster health stats
-  cluster_health = false
-
-  ## Adjust cluster_health_level when you want to also obtain detailed health stats
-  ## The options are
-  ##  - indices (default)
-  ##  - cluster
-  # cluster_health_level = "indices"
-
-  ## Set cluster_stats to true when you want to also obtain cluster stats.
-  cluster_stats = false
-
-  ## Only gather cluster_stats from the master node. To work this require local = true
-  cluster_stats_only_from_master = true
-
-  ## Indices to collect; can be one or more indices names or _all
-  ## Use of wildcards is allowed. Use a wildcard at the end to retrieve index names that end with a changing value, like a date.
-  indices_include = ["_all"]
-
-  ## One of "shards", "cluster", "indices"
-  indices_level = "shards"
-
-  ## node_stats is a list of sub-stats that you want to have gathered. Valid options
-  ## are "indices", "os", "process", "jvm", "thread_pool", "fs", "transport", "http",
-  ## "breaker". Per default, all stats are gathered.
-  # node_stats = ["jvm", "http"]
-
-  ## HTTP Basic Authentication username and password.
-  # username = ""
-  # password = ""
-
-  ## Optional TLS Config
-  # tls_ca = "/etc/telegraf/ca.pem"
-  # tls_cert = "/etc/telegraf/cert.pem"
-  # tls_key = "/etc/telegraf/key.pem"
-  ## Use TLS but skip chain & host verification
-  # insecure_skip_verify = false
-
-  ## Sets the number of most recent indices to return for indices that are configured with a date-stamped suffix.
-  ## Each 'indices_include' entry ending with a wildcard (*) or glob matching pattern will group together all indices that match it, and sort them
-  ## by the date or number after the wildcard. Metrics then are gathered for only the 'num_most_recent_indices' amount of most recent indices.
-  # num_most_recent_indices = 0
-`
-
 // Elasticsearch is a plugin to read stats from one or many Elasticsearch
 // servers.
 type Elasticsearch struct {
@@ -214,16 +162,10 @@ func mapShardStatusToCode(s string) int {
 	return 0
 }
 
-// SampleConfig returns sample configuration for this plugin.
-func (e *Elasticsearch) SampleConfig() string {
+func (*Elasticsearch) SampleConfig() string {
 	return sampleConfig
 }
 
-// Description returns the plugin description.
-func (e *Elasticsearch) Description() string {
-	return "Read stats from one or more Elasticsearch servers or clusters"
-}
-
 // Init the plugin.
 func (e *Elasticsearch) Init() error {
 	// Compile the configured indexes to match for sorting.
@@ -702,7 +644,7 @@ func (e *Elasticsearch) getCatMaster(url string) (string, error) {
 		// future calls.
 		return "", fmt.Errorf("elasticsearch: Unable to retrieve master node information. API responded with status-code %d, expected %d", r.StatusCode, http.StatusOK)
 	}
-	response, err := ioutil.ReadAll(r.Body)
+	response, err := io.ReadAll(r.Body)
 
 	if err != nil {
 		return "", err
