@@ -1,7 +1,9 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package httpjson
 
 import (
 	"bytes"
+	_ "embed"
 	"fmt"
 	"io"
 	"net/http"
@@ -17,6 +19,10 @@ import (
 	"github.com/influxdata/telegraf/plugins/parsers"
 )
 
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//go:embed sample.conf
+var sampleConfig string
+
 var (
 	utf8BOM = []byte("\xef\xbb\xbf")
 )
@@ -66,55 +72,10 @@ func (c *RealHTTPClient) HTTPClient() *http.Client {
 	return c.client
 }
 
-var sampleConfig = `
-  ## NOTE This plugin only reads numerical measurements, strings and booleans
-  ## will be ignored.
-
-  ## URL of each server in the service's cluster
-  servers = [
-    "http://localhost:9999/stats/",
-    "http://localhost:9998/stats/",
-  ]
-  ## Set response_timeout (default 5 seconds)
-  response_timeout = "5s"
-
-  ## HTTP method to use: GET or POST (case-sensitive)
-  method = "GET"
-
-  ## List of tag names to extract from top-level of JSON server response
-  # tag_keys = [
-  #   "my_tag_1",
-  #   "my_tag_2"
-  # ]
-
-  ## Optional TLS Config
-  # tls_ca = "/etc/telegraf/ca.pem"
-  # tls_cert = "/etc/telegraf/cert.pem"
-  # tls_key = "/etc/telegraf/key.pem"
-  ## Use TLS but skip chain & host verification
-  # insecure_skip_verify = false
-
-  ## HTTP parameters (all values must be strings).  For "GET" requests, data
-  ## will be included in the query.  For "POST" requests, data will be included
-  ## in the request body as "x-www-form-urlencoded".
-  # [inputs.httpjson.parameters]
-  #   event_type = "cpu_spike"
-  #   threshold = "0.75"
-
-  ## HTTP Headers (all values must be strings)
-  # [inputs.httpjson.headers]
-  #   X-Auth-Token = "my-xauth-token"
-  #   apiVersion = "v1"
-`
-
-func (h *HTTPJSON) SampleConfig() string {
+func (*HTTPJSON) SampleConfig() string {
 	return sampleConfig
 }
 
-func (h *HTTPJSON) Description() string {
-	return "Read flattened metrics from one or more JSON HTTP endpoints"
-}
-
 // Gathers data for all servers.
 func (h *HTTPJSON) Gather(acc telegraf.Accumulator) error {
 	var wg sync.WaitGroup
