@@ -1,7 +1,9 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package raindrops
 
 import (
 	"bufio"
+	_ "embed"
 	"fmt"
 	"net"
 	"net/http"
@@ -15,24 +17,19 @@ import (
 	"github.com/influxdata/telegraf/plugins/inputs"
 )
 
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//go:embed sample.conf
+var sampleConfig string
+
 type Raindrops struct {
 	Urls       []string
 	httpClient *http.Client
 }
 
-var sampleConfig = `
-  ## An array of raindrops middleware URI to gather stats.
-  urls = ["http://localhost:8080/_raindrops"]
-`
-
-func (r *Raindrops) SampleConfig() string {
+func (*Raindrops) SampleConfig() string {
 	return sampleConfig
 }
 
-func (r *Raindrops) Description() string {
-	return "Read raindrops stats (raindrops - real-time stats for preforking Rack servers)"
-}
-
 func (r *Raindrops) Gather(acc telegraf.Accumulator) error {
 	var wg sync.WaitGroup
 
@@ -116,7 +113,6 @@ func (r *Raindrops) gatherURL(addr *url.URL, acc telegraf.Accumulator) error {
 		}
 		activeLineStr, activeErr = buf.ReadString('\n')
 		if activeErr != nil {
-			iterate = false
 			break
 		}
 		if strings.Compare(activeLineStr, "\n") == 0 {
@@ -154,7 +150,7 @@ func (r *Raindrops) gatherURL(addr *url.URL, acc telegraf.Accumulator) error {
 		}
 		acc.AddFields("raindrops_listen", lis, tags)
 	}
-	return nil
+	return nil //nolint:nilerr // nil returned on purpose
 }
 
 // Get tag(s) for the raindrops calling/writing plugin
