@@ -1,6 +1,8 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package snmp_legacy
 
 import (
+	_ "embed"
 	"log"
 	"net"
 	"os"
@@ -14,6 +16,10 @@ import (
 	"github.com/influxdata/telegraf/plugins/inputs"
 )
 
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//go:embed sample.conf
+var sampleConfig string
+
 // Snmp is a snmp plugin
 type Snmp struct {
 	Host              []Host
@@ -117,119 +123,6 @@ type Node struct {
 	subnodes map[string]Node
 }
 
-var sampleConfig = `
-  ## Use 'oids.txt' file to translate oids to names
-  ## To generate 'oids.txt' you need to run:
-  ##   snmptranslate -m all -Tz -On | sed -e 's/"//g' > /tmp/oids.txt
-  ## Or if you have an other MIB folder with custom MIBs
-  ##   snmptranslate -M /mycustommibfolder -Tz -On -m all | sed -e 's/"//g' > oids.txt
-  snmptranslate_file = "/tmp/oids.txt"
-  [[inputs.snmp.host]]
-    address = "192.168.2.2:161"
-    # SNMP community
-    community = "public" # default public
-    # SNMP version (1, 2 or 3)
-    # Version 3 not supported yet
-    version = 2 # default 2
-    # SNMP response timeout
-    timeout = 2.0 # default 2.0
-    # SNMP request retries
-    retries = 2 # default 2
-    # Which get/bulk do you want to collect for this host
-    collect = ["mybulk", "sysservices", "sysdescr"]
-    # Simple list of OIDs to get, in addition to "collect"
-    get_oids = []
-
-  [[inputs.snmp.host]]
-    address = "192.168.2.3:161"
-    community = "public"
-    version = 2
-    timeout = 2.0
-    retries = 2
-    collect = ["mybulk"]
-    get_oids = [
-        "ifNumber",
-        ".1.3.6.1.2.1.1.3.0",
-    ]
-
-  [[inputs.snmp.get]]
-    name = "ifnumber"
-    oid = "ifNumber"
-
-  [[inputs.snmp.get]]
-    name = "interface_speed"
-    oid = "ifSpeed"
-    instance = "0"
-
-  [[inputs.snmp.get]]
-    name = "sysuptime"
-    oid = ".1.3.6.1.2.1.1.3.0"
-    unit = "second"
-
-  [[inputs.snmp.bulk]]
-    name = "mybulk"
-    max_repetition = 127
-    oid = ".1.3.6.1.2.1.1"
-
-  [[inputs.snmp.bulk]]
-    name = "ifoutoctets"
-    max_repetition = 127
-    oid = "ifOutOctets"
-
-  [[inputs.snmp.host]]
-    address = "192.168.2.13:161"
-    #address = "127.0.0.1:161"
-    community = "public"
-    version = 2
-    timeout = 2.0
-    retries = 2
-    #collect = ["mybulk", "sysservices", "sysdescr", "systype"]
-    collect = ["sysuptime" ]
-    [[inputs.snmp.host.table]]
-      name = "iftable3"
-      include_instances = ["enp5s0", "eth1"]
-
-  # SNMP TABLEs
-  # table without mapping neither subtables
-  [[inputs.snmp.table]]
-    name = "iftable1"
-    oid = ".1.3.6.1.2.1.31.1.1.1"
-
-  # table without mapping but with subtables
-  [[inputs.snmp.table]]
-    name = "iftable2"
-    oid = ".1.3.6.1.2.1.31.1.1.1"
-    sub_tables = [".1.3.6.1.2.1.2.2.1.13"]
-
-  # table with mapping but without subtables
-  [[inputs.snmp.table]]
-    name = "iftable3"
-    oid = ".1.3.6.1.2.1.31.1.1.1"
-    # if empty. get all instances
-    mapping_table = ".1.3.6.1.2.1.31.1.1.1.1"
-    # if empty, get all subtables
-
-  # table with both mapping and subtables
-  [[inputs.snmp.table]]
-    name = "iftable4"
-    oid = ".1.3.6.1.2.1.31.1.1.1"
-    # if empty get all instances
-    mapping_table = ".1.3.6.1.2.1.31.1.1.1.1"
-    # if empty get all subtables
-    # sub_tables could be not "real subtables"
-    sub_tables=[".1.3.6.1.2.1.2.2.1.13", "bytes_recv", "bytes_send"]
-`
-
-// SampleConfig returns sample configuration message
-func (s *Snmp) SampleConfig() string {
-	return sampleConfig
-}
-
-// Description returns description of Zookeeper plugin
-func (s *Snmp) Description() string {
-	return `DEPRECATED! PLEASE USE inputs.snmp INSTEAD.`
-}
-
 func fillnode(parentNode Node, oidName string, ids []string) {
 	// ids = ["1", "3", "6", ...]
 	id, ids := ids[0], ids[1:]
@@ -277,6 +170,10 @@ func findNodeName(node Node, ids []string) (oidName string, instance string) {
 	return node.name, ""
 }
 
+func (*Snmp) SampleConfig() string {
+	return sampleConfig
+}
+
 func (s *Snmp) Gather(acc telegraf.Accumulator) error {
 	// TODO put this in cache on first run
 	// Create subtables mapping
