@@ -3,7 +3,6 @@ package snmp_trap
 import (
 	"fmt"
 	"net"
-	"path/filepath"
 	"strconv"
 	"strings"
 	"testing"
@@ -17,6 +16,28 @@ import (
 	"github.com/influxdata/telegraf/testutil"
 )
 
+type entry struct {
+	oid string
+	e   snmp.MibEntry
+}
+
+type testTranslator struct {
+	entries []entry
+}
+
+func (t *testTranslator) lookup(input string) (snmp.MibEntry, error) {
+	for _, entry := range t.entries {
+		if input == entry.oid {
+			return snmp.MibEntry{MibName: entry.e.MibName, OidText: entry.e.OidText}, nil
+		}
+	}
+	return snmp.MibEntry{}, fmt.Errorf("unexpected oid")
+}
+
+func newTestTranslator(entries []entry) *testTranslator {
+	return &testTranslator{entries: entries}
+}
+
 func newMsgFlagsV3(secLevel string) gosnmp.SnmpV3MsgFlags {
 	var msgFlags gosnmp.SnmpV3MsgFlags
 	switch strings.ToLower(secLevel) {
@@ -131,11 +152,6 @@ func TestReceiveTrap(t *testing.T) {
 	now := uint32(123123123)
 	fakeTime := time.Unix(456456456, 456)
 
-	type entry struct {
-		oid string
-		e   snmp.MibEntry
-	}
-
 	// If the first pdu isn't type TimeTicks, gosnmp.SendTrap() will
 	// prepend one with time.Now()
 	var tests = []struct {
@@ -1261,14 +1277,6 @@ func TestReceiveTrap(t *testing.T) {
 				timeFunc: func() time.Time {
 					return fakeTime
 				},
-				lookupFunc: func(input string) (snmp.MibEntry, error) {
-					for _, entry := range tt.entries {
-						if input == entry.oid {
-							return snmp.MibEntry{MibName: entry.e.MibName, OidText: entry.e.OidText}, nil
-						}
-					}
-					return snmp.MibEntry{}, fmt.Errorf("unexpected oid")
-				},
 				//if cold start be answer otherwise err
 				Log:          testutil.Logger{},
 				Version:      tt.version.String(),
@@ -1278,10 +1286,14 @@ func TestReceiveTrap(t *testing.T) {
 				AuthPassword: tt.authPass,
 				PrivProtocol: tt.privProto,
 				PrivPassword: tt.privPass,
+				Translator:   "netsnmp",
 			}
 
 			require.NoError(t, s.Init())
 
+			//inject test translator
+			s.translator = newTestTranslator(tt.entries)
+
 			var acc testutil.Accumulator
 			require.Nil(t, s.Start(&acc))
 			defer s.Stop()
@@ -1312,96 +1324,3 @@ func TestReceiveTrap(t *testing.T) {
 		})
 	}
 }
-
-func TestGosmiSingleMib(t *testing.T) {
-	// We would prefer to specify port 0 and let the network
-	// stack choose an unused port for us but TrapListener
-	// doesn't have a way to return the autoselected port.
-	// Instead, we'll use an unusual port and hope it's
-	// unused.
-	const port = 12399
-
-	// Hook into the trap handler so the test knows when the
-	// trap has been received
-	received := make(chan int)
-	wrap := func(f gosnmp.TrapHandlerFunc) gosnmp.TrapHandlerFunc {
-		return func(p *gosnmp.SnmpPacket, a *net.UDPAddr) {
-			f(p, a)
-			received <- 0
-		}
-	}
-
-	fakeTime := time.Unix(456456456, 456)
-	now := uint32(123123123)
-
-	testDataPath, err := filepath.Abs("./testdata")
-	require.NoError(t, err)
-
-	trap := gosnmp.SnmpTrap{
-		Variables: []gosnmp.SnmpPDU{
-			{
-				Name:  ".1.3.6.1.2.1.1.3.0",
-				Type:  gosnmp.TimeTicks,
-				Value: now,
-			},
-			{
-				Name:  ".1.3.6.1.6.3.1.1.4.1.0", // SNMPv2-MIB::snmpTrapOID.0
-				Type:  gosnmp.ObjectIdentifier,
-				Value: ".1.3.6.1.6.3.1.1.5.1", // coldStart
-			},
-		},
-	}
-
-	metrics := []telegraf.Metric{
-		testutil.MustMetric(
-			"snmp_trap", // name
-			map[string]string{ // tags
-				"oid":       ".1.3.6.1.6.3.1.1.5.1",
-				"name":      "coldStart",
-				"mib":       "SNMPv2-MIB",
-				"version":   "2c",
-				"source":    "127.0.0.1",
-				"community": "public",
-			},
-			map[string]interface{}{ // fields
-				"sysUpTimeInstance": now,
-			},
-			fakeTime,
-		),
-	}
-
-	// Set up the service input plugin
-	s := &SnmpTrap{
-		ServiceAddress:     "udp://:" + strconv.Itoa(port),
-		makeHandlerWrapper: wrap,
-		timeFunc: func() time.Time {
-			return fakeTime
-		},
-		lookupFunc: snmp.TrapLookup,
-		Log:        testutil.Logger{},
-		Version:    "2c",
-		Path:       []string{testDataPath},
-	}
-	require.NoError(t, s.Init())
-
-	var acc testutil.Accumulator
-	require.Nil(t, s.Start(&acc))
-	defer s.Stop()
-
-	goSNMP := newGoSNMP(gosnmp.Version2c, port)
-
-	// Send the trap
-	sendTrap(t, goSNMP, trap)
-
-	// Wait for trap to be received
-	select {
-	case <-received:
-	case <-time.After(2 * time.Second):
-		t.Fatal("timed out waiting for trap to be received")
-	}
-
-	// Verify plugin output
-	testutil.RequireMetricsEqual(t,
-		metrics, acc.GetTelegrafMetrics(),
-		testutil.SortMetrics())
-}
