@@ -1,9 +1,11 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package zookeeper
 
 import (
 	"bufio"
 	"context"
 	"crypto/tls"
+	_ "embed"
 	"fmt"
 	"net"
 	"regexp"
@@ -17,6 +19,10 @@ import (
 	"github.com/influxdata/telegraf/plugins/inputs"
 )
 
+// DO NOT REMOVE THE NEXT TWO LINES! This is required to embed the sampleConfig data.
+//go:embed sample.conf
+var sampleConfig string
+
 var zookeeperFormatRE = regexp.MustCompile(`^zk_(\w[\w\.\-]*)\s+([\w\.\-]+)`)
 
 // Zookeeper is a zookeeper plugin
@@ -32,38 +38,8 @@ type Zookeeper struct {
 	tlsConfig   *tls.Config
 }
 
-var sampleConfig = `
-  ## An array of address to gather stats about. Specify an ip or hostname
-  ## with port. ie localhost:2181, 10.0.0.1:2181, etc.
-
-  ## If no servers are specified, then localhost is used as the host.
-  ## If no port is specified, 2181 is used
-  servers = [":2181"]
-
-  ## Timeout for metric collections from all servers.  Minimum timeout is "1s".
-  # timeout = "5s"
-
-  ## Optional TLS Config
-  # enable_tls = true
-  # tls_ca = "/etc/telegraf/ca.pem"
-  # tls_cert = "/etc/telegraf/cert.pem"
-  # tls_key = "/etc/telegraf/key.pem"
-  ## If false, skip chain & host verification
-  # insecure_skip_verify = true
-`
-
 var defaultTimeout = 5 * time.Second
 
-// SampleConfig returns sample configuration message
-func (z *Zookeeper) SampleConfig() string {
-	return sampleConfig
-}
-
-// Description returns description of Zookeeper plugin
-func (z *Zookeeper) Description() string {
-	return `Reads 'mntr' stats from one or many zookeeper servers`
-}
-
 func (z *Zookeeper) dial(ctx context.Context, addr string) (net.Conn, error) {
 	var dialer net.Dialer
 	if z.EnableTLS || z.EnableSSL {
@@ -76,6 +52,10 @@ func (z *Zookeeper) dial(ctx context.Context, addr string) (net.Conn, error) {
 	return dialer.DialContext(ctx, "tcp", addr)
 }
 
+func (*Zookeeper) SampleConfig() string {
+	return sampleConfig
+}
+
 // Gather reads stats from all configured servers accumulates stats
 func (z *Zookeeper) Gather(acc telegraf.Accumulator) error {
 	ctx := context.Background()
