@@ -1,6 +1,7 @@
 package xpath
 
 import (
+	"math"
 	"os"
 	"path/filepath"
 	"strings"
@@ -125,7 +126,12 @@ func TestParseInvalidXML(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			parser := &Parser{Configs: tt.configs, DefaultTags: tt.defaultTags, Log: testutil.Logger{Name: "parsers.xml"}}
+			parser := &Parser{
+				DefaultMetricName: "xml",
+				Configs:           tt.configs,
+				DefaultTags:       tt.defaultTags,
+				Log:               testutil.Logger{Name: "parsers.xml"},
+			}
 			require.NoError(t, parser.Init())
 
 			_, err := parser.ParseLine(tt.input)
@@ -148,8 +154,7 @@ func TestInvalidTypeQueriesFail(t *testing.T) {
 			input: singleMetricValuesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix",
+					Timestamp: "/Device_1/Timestamp_unix",
 					FieldsInt: map[string]string{
 						"a": "/Device_1/value_string",
 					},
@@ -162,7 +167,12 @@ func TestInvalidTypeQueriesFail(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			parser := &Parser{Configs: tt.configs, DefaultTags: tt.defaultTags, Log: testutil.Logger{Name: "parsers.xml"}}
+			parser := &Parser{
+				DefaultMetricName: "xml",
+				Configs:           tt.configs,
+				DefaultTags:       tt.defaultTags,
+				Log:               testutil.Logger{Name: "parsers.xml"},
+			}
 			require.NoError(t, parser.Init())
 
 			_, err := parser.ParseLine(tt.input)
@@ -185,8 +195,7 @@ func TestInvalidTypeQueries(t *testing.T) {
 			input: singleMetricValuesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix",
+					Timestamp: "/Device_1/Timestamp_unix",
 					Fields: map[string]string{
 						"a": "number(/Device_1/value_string)",
 					},
@@ -197,7 +206,7 @@ func TestInvalidTypeQueries(t *testing.T) {
 				"test",
 				map[string]string{},
 				map[string]interface{}{
-					"a": float64(0),
+					"a": math.NaN(),
 				},
 				time.Unix(1577923199, 0),
 			),
@@ -207,8 +216,7 @@ func TestInvalidTypeQueries(t *testing.T) {
 			input: singleMetricValuesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix",
+					Timestamp: "/Device_1/Timestamp_unix",
 					Fields: map[string]string{
 						"a": "boolean(/Device_1/value_string)",
 					},
@@ -228,7 +236,12 @@ func TestInvalidTypeQueries(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			parser := &Parser{Configs: tt.configs, DefaultTags: tt.defaultTags, Log: testutil.Logger{Name: "parsers.xml"}}
+			parser := &Parser{
+				DefaultMetricName: "test",
+				Configs:           tt.configs,
+				DefaultTags:       tt.defaultTags,
+				Log:               testutil.Logger{Name: "parsers.xml"},
+			}
 			require.NoError(t, parser.Init())
 
 			actual, err := parser.ParseLine(tt.input)
@@ -252,8 +265,7 @@ func TestParseTimestamps(t *testing.T) {
 			input: singleMetricValuesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix",
+					Timestamp: "/Device_1/Timestamp_unix",
 				},
 			},
 			defaultTags: map[string]string{},
@@ -269,9 +281,8 @@ func TestParseTimestamps(t *testing.T) {
 			input: singleMetricValuesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix",
-					TimestampFmt:      "unix",
+					Timestamp:    "/Device_1/Timestamp_unix",
+					TimestampFmt: "unix",
 				},
 			},
 			defaultTags: map[string]string{},
@@ -287,9 +298,8 @@ func TestParseTimestamps(t *testing.T) {
 			input: singleMetricValuesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix_ms",
-					TimestampFmt:      "unix_ms",
+					Timestamp:    "/Device_1/Timestamp_unix_ms",
+					TimestampFmt: "unix_ms",
 				},
 			},
 			defaultTags: map[string]string{},
@@ -305,9 +315,8 @@ func TestParseTimestamps(t *testing.T) {
 			input: singleMetricValuesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix_us",
-					TimestampFmt:      "unix_us",
+					Timestamp:    "/Device_1/Timestamp_unix_us",
+					TimestampFmt: "unix_us",
 				},
 			},
 			defaultTags: map[string]string{},
@@ -323,9 +332,8 @@ func TestParseTimestamps(t *testing.T) {
 			input: singleMetricValuesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix_ns",
-					TimestampFmt:      "unix_ns",
+					Timestamp:    "/Device_1/Timestamp_unix_ns",
+					TimestampFmt: "unix_ns",
 				},
 			},
 			defaultTags: map[string]string{},
@@ -341,9 +349,8 @@ func TestParseTimestamps(t *testing.T) {
 			input: singleMetricValuesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_iso",
-					TimestampFmt:      "2006-01-02T15:04:05Z",
+					Timestamp:    "/Device_1/Timestamp_iso",
+					TimestampFmt: "2006-01-02T15:04:05Z",
 				},
 			},
 			defaultTags: map[string]string{},
@@ -358,7 +365,12 @@ func TestParseTimestamps(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			parser := &Parser{Configs: tt.configs, DefaultTags: tt.defaultTags, Log: testutil.Logger{Name: "parsers.xml"}}
+			parser := &Parser{
+				DefaultMetricName: "test",
+				Configs:           tt.configs,
+				DefaultTags:       tt.defaultTags,
+				Log:               testutil.Logger{Name: "parsers.xml"},
+			}
 			require.NoError(t, parser.Init())
 
 			actual, err := parser.ParseLine(tt.input)
@@ -382,8 +394,7 @@ func TestParseSingleValues(t *testing.T) {
 			input: singleMetricValuesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix",
+					Timestamp: "/Device_1/Timestamp_unix",
 					Fields: map[string]string{
 						"a": "/Device_1/value_int",
 						"b": "/Device_1/value_float",
@@ -410,8 +421,7 @@ func TestParseSingleValues(t *testing.T) {
 			input: singleMetricValuesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix",
+					Timestamp: "/Device_1/Timestamp_unix",
 					Fields: map[string]string{
 						"a": "number(Device_1/value_int)",
 						"b": "number(/Device_1/value_float)",
@@ -438,8 +448,7 @@ func TestParseSingleValues(t *testing.T) {
 			input: singleMetricValuesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix",
+					Timestamp: "/Device_1/Timestamp_unix",
 					Fields: map[string]string{
 						"b": "number(/Device_1/value_float)",
 						"c": "boolean(/Device_1/value_bool)",
@@ -468,8 +477,7 @@ func TestParseSingleValues(t *testing.T) {
 			input: singleMetricValuesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix",
+					Timestamp: "/Device_1/Timestamp_unix",
 					Fields: map[string]string{
 						"x": "substring-before(/Device_1/value_position, ';')",
 						"y": "substring-after(/Device_1/value_position, ';')",
@@ -492,8 +500,7 @@ func TestParseSingleValues(t *testing.T) {
 			input: singleMetricValuesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix",
+					Timestamp: "/Device_1/Timestamp_unix",
 					Fields: map[string]string{
 						"x": "number(substring-before(/Device_1/value_position, ';'))",
 						"y": "number(substring-after(/Device_1/value_position, ';'))",
@@ -516,8 +523,7 @@ func TestParseSingleValues(t *testing.T) {
 			input: singleMetricValuesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix",
+					Timestamp: "/Device_1/Timestamp_unix",
 					FieldsInt: map[string]string{
 						"x": "substring-before(/Device_1/value_position, ';')",
 						"y": "substring-after(/Device_1/value_position, ';')",
@@ -540,8 +546,7 @@ func TestParseSingleValues(t *testing.T) {
 			input: singleMetricValuesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix",
+					Timestamp: "/Device_1/Timestamp_unix",
 					Tags: map[string]string{
 						"state": "/Device_1/State",
 						"name":  "substring-after(/Device_1/Name, ' ')",
@@ -563,7 +568,12 @@ func TestParseSingleValues(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			parser := &Parser{Configs: tt.configs, DefaultTags: tt.defaultTags, Log: testutil.Logger{Name: "parsers.xml"}}
+			parser := &Parser{
+				DefaultMetricName: "test",
+				Configs:           tt.configs,
+				DefaultTags:       tt.defaultTags,
+				Log:               testutil.Logger{Name: "parsers.xml"},
+			}
 			require.NoError(t, parser.Init())
 
 			actual, err := parser.ParseLine(tt.input)
@@ -587,8 +597,7 @@ func TestParseSingleAttributes(t *testing.T) {
 			input: singleMetricAttributesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix/@value",
+					Timestamp: "/Device_1/Timestamp_unix/@value",
 				},
 			},
 			defaultTags: map[string]string{},
@@ -604,9 +613,8 @@ func TestParseSingleAttributes(t *testing.T) {
 			input: singleMetricAttributesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_iso/@value",
-					TimestampFmt:      "2006-01-02T15:04:05Z",
+					Timestamp:    "/Device_1/Timestamp_iso/@value",
+					TimestampFmt: "2006-01-02T15:04:05Z",
 				},
 			},
 			defaultTags: map[string]string{},
@@ -622,8 +630,7 @@ func TestParseSingleAttributes(t *testing.T) {
 			input: singleMetricAttributesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix/@value",
+					Timestamp: "/Device_1/Timestamp_unix/@value",
 					Fields: map[string]string{
 						"a": "/Device_1/attr_int/@_",
 						"b": "/Device_1/attr_float/@_",
@@ -650,8 +657,7 @@ func TestParseSingleAttributes(t *testing.T) {
 			input: singleMetricAttributesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix/@value",
+					Timestamp: "/Device_1/Timestamp_unix/@value",
 					Fields: map[string]string{
 						"a": "number(/Device_1/attr_int/@_)",
 						"b": "number(/Device_1/attr_float/@_)",
@@ -678,8 +684,7 @@ func TestParseSingleAttributes(t *testing.T) {
 			input: singleMetricAttributesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix/@value",
+					Timestamp: "/Device_1/Timestamp_unix/@value",
 					Fields: map[string]string{
 						"b": "number(/Device_1/attr_float/@_)",
 						"c": "boolean(/Device_1/attr_bool/@_)",
@@ -708,8 +713,7 @@ func TestParseSingleAttributes(t *testing.T) {
 			input: singleMetricAttributesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix/@value",
+					Timestamp: "/Device_1/Timestamp_unix/@value",
 					Fields: map[string]string{
 						"name": "substring-after(/Device_1/Name/@value, ' ')",
 					},
@@ -730,8 +734,7 @@ func TestParseSingleAttributes(t *testing.T) {
 			input: singleMetricAttributesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix/@value",
+					Timestamp: "/Device_1/Timestamp_unix/@value",
 					Tags: map[string]string{
 						"state": "/Device_1/State/@_",
 						"name":  "substring-after(/Device_1/Name/@value, ' ')",
@@ -754,8 +757,7 @@ func TestParseSingleAttributes(t *testing.T) {
 			input: singleMetricAttributesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Device_1/Timestamp_unix/@value",
+					Timestamp: "/Device_1/Timestamp_unix/@value",
 					Fields: map[string]string{
 						"a": "/Device_1/attr_bool_numeric/@_ = 1",
 					},
@@ -775,7 +777,12 @@ func TestParseSingleAttributes(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			parser := &Parser{Configs: tt.configs, DefaultTags: tt.defaultTags, Log: testutil.Logger{Name: "parsers.xml"}}
+			parser := &Parser{
+				DefaultMetricName: "test",
+				Configs:           tt.configs,
+				DefaultTags:       tt.defaultTags,
+				Log:               testutil.Logger{Name: "parsers.xml"},
+			}
 			require.NoError(t, parser.Init())
 
 			actual, err := parser.ParseLine(tt.input)
@@ -799,8 +806,7 @@ func TestParseMultiValues(t *testing.T) {
 			input: singleMetricMultiValuesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Timestamp/@value",
+					Timestamp: "/Timestamp/@value",
 					Fields: map[string]string{
 						"a": "number(/Device/Value[1])",
 						"b": "number(/Device/Value[2])",
@@ -831,8 +837,7 @@ func TestParseMultiValues(t *testing.T) {
 			input: singleMetricMultiValuesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Timestamp:         "/Timestamp/@value",
+					Timestamp: "/Timestamp/@value",
 					FieldsInt: map[string]string{
 						"a": "/Device/Value[1]",
 						"b": "/Device/Value[2]",
@@ -862,7 +867,12 @@ func TestParseMultiValues(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			parser := &Parser{Configs: tt.configs, DefaultTags: tt.defaultTags, Log: testutil.Logger{Name: "parsers.xml"}}
+			parser := &Parser{
+				DefaultMetricName: "test",
+				Configs:           tt.configs,
+				DefaultTags:       tt.defaultTags,
+				Log:               testutil.Logger{Name: "parsers.xml"},
+			}
 			require.NoError(t, parser.Init())
 
 			actual, err := parser.ParseLine(tt.input)
@@ -886,9 +896,8 @@ func TestParseMultiNodes(t *testing.T) {
 			input: multipleNodesXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					Selection:         "/Device",
-					Timestamp:         "/Timestamp/@value",
+					Selection: "/Device",
+					Timestamp: "/Timestamp/@value",
 					Fields: map[string]string{
 						"value":  "number(Value)",
 						"active": "Active = 1",
@@ -975,7 +984,12 @@ func TestParseMultiNodes(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			parser := &Parser{Configs: tt.configs, DefaultTags: tt.defaultTags, Log: testutil.Logger{Name: "parsers.xml"}}
+			parser := &Parser{
+				DefaultMetricName: "test",
+				Configs:           tt.configs,
+				DefaultTags:       tt.defaultTags,
+				Log:               testutil.Logger{Name: "parsers.xml"},
+			}
 			require.NoError(t, parser.Init())
 
 			actual, err := parser.Parse([]byte(tt.input))
@@ -999,9 +1013,8 @@ func TestParseMetricQuery(t *testing.T) {
 			input: metricNameQueryXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					MetricQuery:       "name(/Device_1/Metric/@*[1])",
-					Timestamp:         "/Device_1/Timestamp_unix",
+					MetricQuery: "name(/Device_1/Metric/@*[1])",
+					Timestamp:   "/Device_1/Timestamp_unix",
 					Fields: map[string]string{
 						"value": "/Device_1/Metric/@*[1]",
 					},
@@ -1022,9 +1035,8 @@ func TestParseMetricQuery(t *testing.T) {
 			input: metricNameQueryXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					MetricQuery:       "'the_metric'",
-					Timestamp:         "/Device_1/Timestamp_unix",
+					MetricQuery: "'the_metric'",
+					Timestamp:   "/Device_1/Timestamp_unix",
 					Fields: map[string]string{
 						"value": "/Device_1/Metric/@*[1]",
 					},
@@ -1044,7 +1056,12 @@ func TestParseMetricQuery(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			parser := &Parser{Configs: tt.configs, DefaultTags: tt.defaultTags, Log: testutil.Logger{Name: "parsers.xml"}}
+			parser := &Parser{
+				DefaultMetricName: "test",
+				Configs:           tt.configs,
+				DefaultTags:       tt.defaultTags,
+				Log:               testutil.Logger{Name: "parsers.xml"},
+			}
 			require.NoError(t, parser.Init())
 
 			actual, err := parser.ParseLine(tt.input)
@@ -1067,9 +1084,8 @@ func TestParseErrors(t *testing.T) {
 			input: metricNameQueryXML,
 			configs: []Config{
 				{
-					MetricDefaultName: "test",
-					MetricQuery:       "arbitrary",
-					Timestamp:         "/Device_1/Timestamp_unix",
+					MetricQuery: "arbitrary",
+					Timestamp:   "/Device_1/Timestamp_unix",
 					Fields: map[string]string{
 						"value": "/Device_1/Metric/@*[1]",
 					},
@@ -1081,7 +1097,12 @@ func TestParseErrors(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			parser := &Parser{Configs: tt.configs, DefaultTags: map[string]string{}, Log: testutil.Logger{Name: "parsers.xml"}}
+			parser := &Parser{
+				DefaultMetricName: "test",
+				Configs:           tt.configs,
+				DefaultTags:       map[string]string{},
+				Log:               testutil.Logger{Name: "parsers.xml"},
+			}
 			require.NoError(t, parser.Init())
 
 			_, err := parser.ParseLine(tt.input)
@@ -1149,7 +1170,12 @@ func TestEmptySelection(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			parser := &Parser{Configs: tt.configs, DefaultTags: map[string]string{}, Log: testutil.Logger{Name: "parsers.xml"}}
+			parser := &Parser{
+				DefaultMetricName: "test",
+				Configs:           tt.configs,
+				DefaultTags:       map[string]string{},
+				Log:               testutil.Logger{Name: "parsers.xml"},
+			}
 			require.NoError(t, parser.Init())
 
 			_, err := parser.Parse([]byte(tt.input))
@@ -1159,6 +1185,79 @@ func TestEmptySelection(t *testing.T) {
 	}
 }
 
+func TestEmptySelectionAllowed(t *testing.T) {
+	var tests = []struct {
+		name    string
+		input   string
+		configs []Config
+	}{
+		{
+			name:  "empty path",
+			input: multipleNodesXML,
+			configs: []Config{
+				{
+					Selection: "/Device/NonExisting",
+					Fields:    map[string]string{"value": "number(Value)"},
+					FieldsInt: map[string]string{"mode": "Value/@mode"},
+					Tags:      map[string]string{},
+				},
+			},
+		},
+		{
+			name:  "empty pattern",
+			input: multipleNodesXML,
+			configs: []Config{
+				{
+					Selection: "//NonExisting",
+					Fields:    map[string]string{"value": "number(Value)"},
+					FieldsInt: map[string]string{"mode": "Value/@mode"},
+					Tags:      map[string]string{},
+				},
+			},
+		},
+		{
+			name:  "empty axis",
+			input: multipleNodesXML,
+			configs: []Config{
+				{
+					Selection: "/Device/child::NonExisting",
+					Fields:    map[string]string{"value": "number(Value)"},
+					FieldsInt: map[string]string{"mode": "Value/@mode"},
+					Tags:      map[string]string{},
+				},
+			},
+		},
+		{
+			name:  "empty predicate",
+			input: multipleNodesXML,
+			configs: []Config{
+				{
+					Selection: "/Device[@NonExisting=true]",
+					Fields:    map[string]string{"value": "number(Value)"},
+					FieldsInt: map[string]string{"mode": "Value/@mode"},
+					Tags:      map[string]string{},
+				},
+			},
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			parser := &Parser{
+				DefaultMetricName:   "xml",
+				Configs:             tt.configs,
+				AllowEmptySelection: true,
+				DefaultTags:         map[string]string{},
+				Log:                 testutil.Logger{Name: "parsers.xml"},
+			}
+			require.NoError(t, parser.Init())
+
+			_, err := parser.Parse([]byte(tt.input))
+			require.NoError(t, err)
+		})
+	}
+}
+
 func TestTestCases(t *testing.T) {
 	var tests = []struct {
 		name     string
@@ -1209,7 +1308,6 @@ func TestTestCases(t *testing.T) {
 			filename := filepath.FromSlash(tt.filename)
 			cfg, header, err := loadTestConfiguration(filename)
 			require.NoError(t, err)
-			cfg.MetricDefaultName = "xml"
 
 			// Load the xml-content
 			input, err := testutil.ParseRawLinesFrom(header, "File:")
@@ -1247,7 +1345,12 @@ func TestTestCases(t *testing.T) {
 			expectedErrors, _ := testutil.ParseRawLinesFrom(header, "Expected Error:")
 
 			// Setup the parser and run it.
+			metricName := "xml"
+			if fileformat != "" {
+				metricName = fileformat
+			}
 			parser := &Parser{
+				DefaultMetricName:   metricName,
 				Format:              fileformat,
 				ProtobufMessageDef:  pbmsgdef,
 				ProtobufMessageType: pbmsgtype,
@@ -1272,6 +1375,7 @@ func TestTestCases(t *testing.T) {
 func TestProtobufImporting(t *testing.T) {
 	// Setup the parser and run it.
 	parser := &Parser{
+		DefaultMetricName:   "xpath_protobuf",
 		Format:              "xpath_protobuf",
 		ProtobufMessageDef:  "person.proto",
 		ProtobufMessageType: "importtest.Person",
