@@ -7,13 +7,13 @@ import (
 	"testing"
 	"time"
 
-	"github.com/influxdata/telegraf"
+	"github.com/stretchr/testify/require"
+
 	"github.com/influxdata/telegraf/config"
 	"github.com/influxdata/telegraf/metric"
 	"github.com/influxdata/telegraf/plugins/parsers/influx"
 	"github.com/influxdata/telegraf/plugins/serializers"
 	"github.com/influxdata/telegraf/testutil"
-	"github.com/stretchr/testify/require"
 )
 
 func TestExternalProcessorWorks(t *testing.T) {
@@ -24,6 +24,7 @@ func TestExternalProcessorWorks(t *testing.T) {
 	require.NoError(t, err)
 	t.Log(exe)
 	e.Command = []string{exe, "-countmultiplier"}
+	e.Environment = []string{"PLUGINS_PROCESSORS_EXECD_MODE=application", "FIELD_NAME=count"}
 	e.RestartDelay = config.Duration(5 * time.Second)
 
 	acc := &testutil.Accumulator{}
@@ -32,7 +33,6 @@ func TestExternalProcessorWorks(t *testing.T) {
 
 	now := time.Now()
 	orig := now
-	metrics := []telegraf.Metric{}
 	for i := 0; i < 10; i++ {
 		m := metric.New("test",
 			map[string]string{
@@ -43,17 +43,16 @@ func TestExternalProcessorWorks(t *testing.T) {
 				"count":      1,
 			},
 			now)
-		metrics = append(metrics, m)
 		now = now.Add(1)
 
-		e.Add(m, acc)
+		require.NoError(t, e.Add(m, acc))
 	}
 
 	acc.Wait(1)
 	require.NoError(t, e.Stop())
 	acc.Wait(9)
 
-	metrics = acc.GetTelegrafMetrics()
+	metrics := acc.GetTelegrafMetrics()
 	m := metrics[0]
 
 	expected := testutil.MustMetric("test",
@@ -86,6 +85,7 @@ func TestParseLinesWithNewLines(t *testing.T) {
 	require.NoError(t, err)
 	t.Log(exe)
 	e.Command = []string{exe, "-countmultiplier"}
+	e.Environment = []string{"PLUGINS_PROCESSORS_EXECD_MODE=application", "FIELD_NAME=count"}
 	e.RestartDelay = config.Duration(5 * time.Second)
 
 	acc := &testutil.Accumulator{}
@@ -105,7 +105,7 @@ func TestParseLinesWithNewLines(t *testing.T) {
 		},
 		now)
 
-	e.Add(m, acc)
+	require.NoError(t, e.Add(m, acc))
 
 	acc.Wait(1)
 	require.NoError(t, e.Stop())
@@ -131,7 +131,8 @@ var countmultiplier = flag.Bool("countmultiplier", false,
 
 func TestMain(m *testing.M) {
 	flag.Parse()
-	if *countmultiplier {
+	runMode := os.Getenv("PLUGINS_PROCESSORS_EXECD_MODE")
+	if *countmultiplier && runMode == "application" {
 		runCountMultiplierProgram()
 		os.Exit(0)
 	}
@@ -140,44 +141,56 @@ func TestMain(m *testing.M) {
 }
 
 func runCountMultiplierProgram() {
+	fieldName := os.Getenv("FIELD_NAME")
 	parser := influx.NewStreamParser(os.Stdin)
 	serializer, _ := serializers.NewInfluxSerializer()
 
 	for {
-		metric, err := parser.Next()
+		m, err := parser.Next()
 		if err != nil {
 			if err == influx.EOF {
 				return // stream ended
 			}
 			if parseErr, isParseError := err.(*influx.ParseError); isParseError {
+				//nolint:errcheck,revive // Test will fail anyway
 				fmt.Fprintf(os.Stderr, "parse ERR %v\n", parseErr)
+				//nolint:revive // os.Exit called intentionally
 				os.Exit(1)
 			}
+			//nolint:errcheck,revive // Test will fail anyway
 			fmt.Fprintf(os.Stderr, "ERR %v\n", err)
+			//nolint:revive // os.Exit called intentionally
 			os.Exit(1)
 		}
 
-		c, found := metric.GetField("count")
+		c, found := m.GetField(fieldName)
 		if !found {
-			fmt.Fprintf(os.Stderr, "metric has no count field\n")
+			//nolint:errcheck,revive // Test will fail anyway
+			fmt.Fprintf(os.Stderr, "metric has no %s field\n", fieldName)
+			//nolint:revive // os.Exit called intentionally
 			os.Exit(1)
 		}
 		switch t := c.(type) {
 		case float64:
 			t *= 2
-			metric.AddField("count", t)
+			m.AddField(fieldName, t)
 		case int64:
 			t *= 2
-			metric.AddField("count", t)
+			m.AddField(fieldName, t)
 		default:
-			fmt.Fprintf(os.Stderr, "count is not an unknown type, it's a %T\n", c)
+			//nolint:errcheck,revive // Test will fail anyway
+			fmt.Fprintf(os.Stderr, "%s is not an unknown type, it's a %T\n", fieldName, c)
+			//nolint:revive // os.Exit called intentionally
 			os.Exit(1)
 		}
-		b, err := serializer.Serialize(metric)
+		b, err := serializer.Serialize(m)
 		if err != nil {
+			//nolint:errcheck,revive // Test will fail anyway
 			fmt.Fprintf(os.Stderr, "ERR %v\n", err)
+			//nolint:revive // os.Exit called intentionally
 			os.Exit(1)
 		}
+		//nolint:errcheck,revive // Test will fail anyway
 		fmt.Fprint(os.Stdout, string(b))
 	}
 }
